@@ -1,36 +1,33 @@
 -- | Compute the debianization of a cabal package.
-{-# LANGUAGE CPP, OverloadedStrings, ScopedTypeVariables #-}
+{-# LANGUAGE CPP, OverloadedStrings, ScopedTypeVariables, TupleSections #-}
 module Debian.Debianize.BuildDependencies
     ( debianBuildDeps
     , debianBuildDepsIndep
     ) where
 
+import Control.Applicative ((<$>))
 import Control.Monad.State (MonadState(get))
 import Control.Monad.Trans (MonadIO)
-import Data.Char (isSpace)
+import Data.Char (isSpace, toLower)
 import Data.Function (on)
 import Data.Lens.Lazy (access, getL)
 import Data.List as List (filter, map, minimumBy, nub)
 import Data.Map as Map (lookup, Map)
 import Data.Maybe (catMaybes, fromMaybe, isJust, isNothing)
-import qualified Data.Set as Set (member)
+import Data.Set as Set (Set, member, toList, fromList, map, fold, union, empty)
 import Data.Version (showVersion, Version)
 import Debian.Debianize.Bundled (builtIn)
 import Debian.Debianize.DebianName (mkPkgName, mkPkgName')
 import Debian.Debianize.Monad as Monad (Atoms, DebT)
 import qualified Debian.Debianize.Types as T (buildDepends, buildDependsIndep, debianNameMap, epochMap, execMap, extraLibMap, missingDependencies, noDocumentationLibrary, noProfilingLibrary)
-import Debian.Debianize.Types.Atoms (EnvSet(dependOS), compilerFlavor, buildEnv)
+import Debian.Debianize.Types.Atoms (EnvSet(dependOS), buildEnv, compilerFlavors)
 import qualified Debian.Debianize.Types.BinaryDebDescription as B (PackageType(Development, Documentation, Profiling))
 import Debian.Debianize.VersionSplits (packageRangesFromVersionSplits)
 import Debian.Orphans ()
 import Debian.Relation (BinPkgName(..), Relation(..), Relations, checkVersionReq)
 import qualified Debian.Relation as D (BinPkgName(BinPkgName), Relation(..), Relations, VersionReq(EEQ, GRE, LTE, SGR, SLT))
 import Debian.Version (DebianVersion, parseDebianVersion)
-#if MIN_VERSION_Cabal(1,21,0)
-import Distribution.Compiler (CompilerFlavor(GHC, GHCJS))
-#else
-import Distribution.Compiler (CompilerFlavor(GHC))
-#endif
+import Distribution.Compiler (CompilerFlavor(..))
 import Distribution.Package (Dependency(..), PackageIdentifier(..), PackageName(PackageName))
 import Distribution.PackageDescription (PackageDescription)
 import Distribution.PackageDescription as Cabal (allBuildInfo, BuildInfo(..), BuildInfo(buildTools, extraLibs, pkgconfigDepends), Executable(..))
@@ -75,47 +72,63 @@ allBuildDepends buildDepends' buildTools' pkgconfigDepends' extraLibs' =
     where
       fixDeps :: Atoms -> [String] -> Relations
       fixDeps atoms xs =
-          concatMap (\ cab -> fromMaybe [[D.Rel (D.BinPkgName ("lib" ++ cab ++ "-dev")) Nothing Nothing]]
+          concatMap (\ cab -> fromMaybe [[D.Rel (D.BinPkgName ("lib" ++ List.map toLower cab ++ "-dev")) Nothing Nothing]]
                                         (Map.lookup cab (getL T.extraLibMap atoms))) xs
 
 -- The haskell-cdbs package contains the hlibrary.mk file with
 -- the rules for building haskell packages.
-debianBuildDeps :: MonadIO m => PackageDescription -> DebT m D.Relations
+debianBuildDeps :: (MonadIO m, Functor m) => PackageDescription -> DebT m D.Relations
 debianBuildDeps pkgDesc =
-    do deb <- get
-       cDeps <- cabalDeps
-       let bDeps = getL T.buildDepends deb
-           prof = not $ getL T.noProfilingLibrary deb
-       cfl <- access compilerFlavor
+    do hcs <- access compilerFlavors
+       let hcTypePairs =
+               fold union empty $
+                  Set.map (\ hc -> Set.map (hc,) $ hcPackageTypes hc) hcs
+       cDeps <- cabalDeps hcTypePairs
+       bDeps <- access T.buildDepends
+       prof <- not <$> access T.noProfilingLibrary
        let xs = nub $ [[D.Rel (D.BinPkgName "debhelper") (Just (D.GRE (parseDebianVersion ("7.0" :: String)))) Nothing],
                        [D.Rel (D.BinPkgName "haskell-devscripts") (Just (D.GRE (parseDebianVersion ("0.8" :: String)))) Nothing],
                        anyrel "cdbs"] ++
-                      (case cfl of
-                        GHC -> [anyrel "ghc"] ++ if prof then [anyrel "ghc-prof"] else []
+                      (if member GHC hcs
+                       then [anyrel "ghc"] ++ if prof
+                                              then [anyrel "ghc-prof"]
+                                              else []
+                       else []) ++
 #if MIN_VERSION_Cabal(1,21,0)
-                        GHCJS -> [anyrel "ghcjs"]
+                      (if member GHCJS hcs then [anyrel "ghcjs"] else []) ++
 #endif
-                        x -> error ("Unsupported compiler flavor: " ++ show x)) ++
-                      bDeps ++
-                      cDeps
+                       bDeps ++
+                       cDeps
        filterMissing xs
     where
-      cabalDeps =
+      cabalDeps hcTypePairs =
           do deps <- allBuildDepends
                           (Cabal.buildDepends pkgDesc ++ concatMap (Cabal.targetBuildDepends . Cabal.buildInfo) (Cabal.executables pkgDesc))
                           (concatMap buildTools . allBuildInfo $ pkgDesc)
                           (concatMap pkgconfigDepends . allBuildInfo $ pkgDesc)
                           (concatMap extraLibs . allBuildInfo $ pkgDesc)
-             mapM buildDependencies (List.filter (not . selfDependency (Cabal.package pkgDesc)) deps) >>= return . concat
+             mapM (buildDependencies hcTypePairs) (List.filter (not . selfDependency (Cabal.package pkgDesc)) deps) >>= return . concat
+      hcPackageTypes :: CompilerFlavor -> Set B.PackageType
+      hcPackageTypes GHC = fromList [B.Development, B.Profiling]
+#if MIN_VERSION_Cabal(1,21,0)
+      hcPackageTypes GHCJS = fromList [B.Development]
+#endif
+      hcPackageTypes hc = error $ "Unsupported compiler flavor: " ++ show hc
 
-debianBuildDepsIndep :: MonadIO m => PackageDescription -> DebT m D.Relations
+
+debianBuildDepsIndep :: (MonadIO m, Functor m) => PackageDescription -> DebT m D.Relations
 debianBuildDepsIndep pkgDesc =
-    do doc <- get >>= return . not . getL T.noDocumentationLibrary
+    do hcs <- access compilerFlavors
+       doc <- get >>= return . not . getL T.noDocumentationLibrary
        bDeps <- get >>= return . getL T.buildDependsIndep
        cDeps <- cabalDeps
-       let xs = if doc
-                then nub $ [anyrel "ghc-doc"] ++ bDeps ++ concat cDeps
-                else []
+       let xs = nub $ if doc
+                      then (if member GHC hcs then [anyrel "ghc-doc"] else []) ++
+#if MIN_VERSION_Cabal(1,21,0)
+                           (if member GHCJS hcs then [anyrel "ghcjs"] else []) ++
+#endif
+                           bDeps ++ concat cDeps
+                      else []
        filterMissing xs
     where
       cabalDeps =
@@ -138,23 +151,22 @@ debianBuildDepsIndep pkgDesc =
 -- | The documentation dependencies for a package include the
 -- documentation package for any libraries which are build
 -- dependencies, so we have access to all the cross references.
-docDependencies :: MonadIO m => Dependency_ -> DebT m D.Relations
-docDependencies (BuildDepends (Dependency name ranges)) = dependencies B.Documentation name ranges
+docDependencies :: (MonadIO m, Functor m) => Dependency_ -> DebT m D.Relations
+docDependencies (BuildDepends (Dependency name ranges)) =
+    do hcs <- access compilerFlavors
+       concat <$> mapM (\ hc -> dependencies hc B.Documentation name ranges) (toList hcs)
 docDependencies _ = return []
 
 -- | The Debian build dependencies for a package include the profiling
 -- libraries and the documentation packages, used for creating cross
 -- references.  Also the packages associated with extra libraries.
-buildDependencies :: MonadIO m => Dependency_ -> DebT m D.Relations
-buildDependencies (BuildDepends (Dependency name ranges)) =
-    do hc <- access compilerFlavor
-       dev <- dependencies B.Development name ranges
-       prof <- if hc == GHC then dependencies B.Profiling name ranges else return []
-       return $ dev ++ prof
-buildDependencies dep@(ExtraLibs _) =
-    do mp <- get >>= return . getL T.execMap
+buildDependencies :: (MonadIO m, Functor m) => Set (CompilerFlavor, B.PackageType) -> Dependency_ -> DebT m D.Relations
+buildDependencies hcTypePairs (BuildDepends (Dependency name ranges)) =
+    concat <$> mapM (\ (hc, typ) -> dependencies hc typ name ranges) (toList hcTypePairs)
+buildDependencies _ dep@(ExtraLibs _) =
+    do mp <- access T.execMap
        return $ concat $ adapt mp dep
-buildDependencies dep =
+buildDependencies _ dep =
     case unboxDependency dep of
       Just (Dependency _name _ranges) ->
           do mp <- get >>= return . getL T.execMap
@@ -199,10 +211,9 @@ anyrel' x = [D.Rel x Nothing Nothing]
 -- | Turn a cabal dependency into debian dependencies.  The result
 -- needs to correspond to a single debian package to be installed,
 -- so we will return just an OrRelation.
-dependencies :: MonadIO m => B.PackageType -> PackageName -> VersionRange -> DebT m Relations
-dependencies typ name cabalRange =
+dependencies :: MonadIO m => CompilerFlavor -> B.PackageType -> PackageName -> VersionRange -> DebT m Relations
+dependencies hc typ name cabalRange =
     do atoms <- get
-       hc <- access compilerFlavor
        -- Compute a list of alternative debian dependencies for
        -- satisfying a cabal dependency.  The only caveat is that
        -- we may need to distribute any "and" dependencies implied
@@ -213,8 +224,8 @@ dependencies typ name cabalRange =
                     -- return the single dependency for the package.
                     Nothing -> [(mkPkgName hc name typ, cabalRange')]
                     -- If there are splits create a list of (debian package name, VersionRange) pairs
-                    Just splits' -> List.map (\ (n, r) -> (mkPkgName' hc n typ, r)) (packageRangesFromVersionSplits splits')
-       mapM convert alts >>= mapM (doBundled typ name) . convert' . canonical . Or . catMaybes
+                    Just splits' -> List.map (\ (n, r) -> (mkPkgName' hc typ n, r)) (packageRangesFromVersionSplits splits')
+       mapM convert alts >>= mapM (doBundled typ name hc) . convert' . canonical . Or . catMaybes
     where
       convert (dname, range) =
           case isNoVersion range''' of
@@ -269,18 +280,18 @@ dependencies typ name cabalRange =
 doBundled :: MonadIO m =>
              B.PackageType
           -> PackageName
+          -> CompilerFlavor
           -> [D.Relation]
           -> DebT m [D.Relation]
-doBundled typ name rels =
+doBundled typ name hc rels =
     mapM doRel rels >>= return . concat
     where
       -- If a library is built into the compiler, this is the debian
       -- package name the compiler will conflict with.
-      comp = D.Rel (compilerPackageName typ) Nothing Nothing
+      comp = D.Rel (compilerPackageName hc typ) Nothing Nothing
       doRel :: MonadIO m => D.Relation -> DebT m [D.Relation]
       doRel rel@(D.Rel dname req _) = do
         -- gver <- access ghcVersion
-        hc <- access compilerFlavor
         splits <- access T.debianNameMap
         root <- access buildEnv >>= return . dependOS
         -- Look at what version of the package is provided by the compiler.
@@ -301,10 +312,17 @@ doBundled typ name rels =
                  -- compiler doesn't conflict with the package's
                  -- debian name.
                  (if isNothing pver || dname /= naiveDebianName || not (conflictsWithHC naiveDebianName) then [rel] else [])
-      compilerPackageName B.Documentation = D.BinPkgName "ghc-doc"
-      compilerPackageName B.Profiling = D.BinPkgName "ghc-prof"
-      compilerPackageName B.Development = D.BinPkgName "ghc"
-      compilerPackageName _ = D.BinPkgName "ghc" -- whatevs
+      compilerPackageName GHC B.Documentation = D.BinPkgName "ghc-doc"
+      compilerPackageName GHC B.Profiling = D.BinPkgName "ghc-prof"
+      compilerPackageName GHC B.Development = D.BinPkgName "ghc"
+      compilerPackageName GHC _ = D.BinPkgName "ghc" -- whatevs
+#if MIN_VERSION_Cabal(1,21,0)
+      compilerPackageName GHCJS B.Documentation = D.BinPkgName "ghcjs"
+      compilerPackageName GHCJS B.Profiling = error "Profiling not supported for GHCJS"
+      compilerPackageName GHCJS B.Development = D.BinPkgName "ghcjs"
+      compilerPackageName GHCJS _ = D.BinPkgName "ghcjs" -- whatevs
+#endif
+      compilerPackageName x _ = error $ "Unsupported compiler flavor: " ++ show x
 
       -- FIXME: we are assuming here that ghc conflicts with all the
       -- library packages it provides but it no longer conflicts with
