@@ -1,5 +1,5 @@
 -- | Compute the debianization of a cabal package.
-{-# LANGUAGE OverloadedStrings, ScopedTypeVariables #-}
+{-# LANGUAGE CPP, OverloadedStrings, ScopedTypeVariables #-}
 module Debian.Debianize.Finalize
     ( debianization
     , finalizeDebianization' -- external use deprecated - used in test script
@@ -15,33 +15,37 @@ import Data.Char (toLower)
 import Data.Digest.Pure.MD5 (md5)
 import Data.Lens.Lazy (access, getL)
 import Data.List as List (intercalate, map, nub, unlines)
-import Data.Map as Map (delete, elems, lookup, map, Map, toList, unionsWith)
+import Data.Map as Map (delete, elems, lookup, Map, toList, insertWith)
 import Data.Maybe (fromMaybe, isJust)
-import Data.Monoid ((<>))
-import Data.Set as Set (difference, filter, fromList, map, null, Set, singleton, toList, union, unions)
+import Data.Monoid ((<>), mempty)
+import Data.Set as Set (difference, filter, fromList, map, null, Set, singleton, toList, union, unions, fold)
 import Data.Set.Extra as Set (mapM_)
 import Data.Text as Text (pack, unlines, unpack)
 import Debian.Changes (ChangeLog(..), ChangeLogEntry(..))
 import Debian.Debianize.BuildDependencies (debianBuildDeps, debianBuildDepsIndep)
 import Debian.Debianize.Changelog (dropFutureEntries)
-import Debian.Debianize.DebianName (debianName)
+import Debian.Debianize.DebianName (debianName, debianNameBase)
 import Debian.Debianize.Goodies (backupAtoms, describe, execAtoms, serverAtoms, siteAtoms, watchAtom)
 import Debian.Debianize.Input (dataDir, inputCabalization, inputChangeLog, inputMaintainer)
 import Debian.Debianize.Monad as Monad (DebT)
 import Debian.Debianize.Options (compileCommandlineArgs, compileEnvironmentArgs)
-import Debian.Debianize.Prelude ((%=), (+++=), (+=), foldEmpty, fromEmpty, fromSingleton, (~=), (~?=))
-import qualified Debian.Debianize.Types as T (apacheSite, backups, binaryArchitectures, binaryPackages, binarySection, breaks, buildDepends, buildDependsIndep, buildDir, builtUsing, changelog, comments, compat, conflicts, debianDescription, debVersion, depends, epochMap, executable, extraDevDeps, extraLibMap, file, install, installCabalExec, installCabalExecTo, installData, installDir, installTo, intermediateFiles, license, link, maintainer, noDocumentationLibrary, noProfilingLibrary, noHoogle, packageDescription, packageType, preDepends, provides, recommends, replaces, revision, rulesFragments, serverInfo, source, sourcePackageName, sourcePriority, sourceSection, suggests, utilsPackageNames, verbosity, watch, website)
-import qualified Debian.Debianize.Types.Atoms as A (InstallFile(execName, sourceDir), showAtoms, compilerFlavor)
-import qualified Debian.Debianize.Types.BinaryDebDescription as B (BinaryDebDescription, package, PackageType(Development, Documentation, Exec, Profiling, Source', Utilities))
+import Debian.Debianize.Prelude ((%=), (+=), foldEmpty, fromEmpty, fromSingleton, (~=), (~?=))
+import qualified Debian.Debianize.Types as T (apacheSite, backups, binaryArchitectures, binaryPackages, binarySection, breaks, buildDepends, buildDependsIndep, buildDir, builtUsing, changelog, comments, compat, conflicts, debianDescription, debVersion, depends, epochMap, executable, extraDevDeps, extraLibMap, file, install, installCabalExec, installData, installDir, installTo, intermediateFiles, license, link, maintainer, noDocumentationLibrary, noProfilingLibrary, noHoogle, packageDescription, packageType, preDepends, provides, recommends, replaces, revision, rulesFragments, serverInfo, source, sourcePackageName, sourcePriority, sourceSection, suggests, utilsPackageNameBase, verbosity, watch, website)
+import qualified Debian.Debianize.Types.Atoms as A (InstallFile(execName, sourceDir), showAtoms, compilerFlavors, Atom(..), atomSet)
+import qualified Debian.Debianize.Types.BinaryDebDescription as B (BinaryDebDescription, package, PackageType(Development, Documentation, Exec, Profiling, Source, HaskellSource, Utilities), PackageType)
+import Debian.Debianize.VersionSplits (DebBase(DebBase))
 import Debian.Orphans ()
 import Debian.Policy (getDebhelperCompatLevel, haskellMaintainer, PackageArchitectures(Any, All), PackagePriority(Optional), Section(..))
 import Debian.Pretty (pretty)
-import Debian.Relation (BinPkgName, BinPkgName(BinPkgName), Relation(Rel), Relations)
+import Debian.Relation (BinPkgName, BinPkgName(BinPkgName), SrcPkgName(SrcPkgName), Relation(Rel), Relations)
 import qualified Debian.Relation as D (BinPkgName(BinPkgName), Relation(..))
 import Debian.Release (parseReleaseName)
 import Debian.Time (getCurrentLocalRFC822Time)
 import Debian.Version (buildDebianVersion, DebianVersion, parseDebianVersion)
 import Distribution.Compiler (CompilerFlavor(GHC))
+#if MIN_VERSION_Cabal(1,21,0)
+import Distribution.Compiler (CompilerFlavor(GHCJS))
+#endif
 import Distribution.Package (Dependency(..), PackageIdentifier(..), PackageName(PackageName))
 import Distribution.PackageDescription (PackageDescription)
 import Distribution.PackageDescription as Cabal (allBuildInfo, BuildInfo(buildable, extraLibs), Executable(buildInfo, exeName))
@@ -83,7 +87,8 @@ finalizeDebianization' =
 --        paramter type to Maybe PackageDescription and propagate down thru code
 finalizeDebianization  :: (MonadIO m, Functor m) => String -> Maybe Int -> DebT m ()
 finalizeDebianization date debhelperCompat =
-    do addExtraLibDependencies
+    do hcs <- Set.toList <$> access A.compilerFlavors
+       List.mapM_ addExtraLibDependencies hcs
        Just pkgDesc <- access T.packageDescription
        T.watch ~?= Just (watchAtom (pkgName $ Cabal.package $ pkgDesc))
        T.sourceSection ~?= Just (MainSection "haskell")
@@ -99,8 +104,8 @@ finalizeDebianization date debhelperCompat =
        access T.serverInfo >>= List.mapM_ (cabalExecBinaryPackage . fst) . Map.toList
        access T.website >>= List.mapM_ (cabalExecBinaryPackage . fst) . Map.toList
        putBuildDeps pkgDesc
-       librarySpecs pkgDesc
-       makeUtilsPackages pkgDesc
+       List.mapM_ (librarySpecs pkgDesc) hcs
+       List.mapM_ (makeUtilsPackage pkgDesc) hcs
        -- Sketchy - I think more things that need expanded could be generated by the code
        -- executed since the last expandAtoms.  Anyway, should be idempotent.
        expandAtoms
@@ -152,18 +157,21 @@ debianEpoch name = get >>= return . Map.lookup name . getL T.epochMap
 -- | Compute and return the debian source package name, based on the
 -- sourcePackageName if it was specified, and constructed from the
 -- cabal name otherwise.
-finalizeSourceName :: Monad m => DebT m ()
-finalizeSourceName =
-    do debName <- debianName B.Source'
-       T.sourcePackageName ~?= Just debName
+finalizeSourceName :: (Monad m, Functor m) => B.PackageType -> DebT m ()
+finalizeSourceName typ =
+    do DebBase debName <- debianNameBase
+       T.sourcePackageName ~?= Just (SrcPkgName (case typ of
+                                                   B.HaskellSource -> "haskell-" ++ debName
+                                                   B.Source -> debName
+                                                   _ -> error $ "finalizeSourceName: " ++ show typ))
 
 finalizeMaintainer :: Monad m => DebT m ()
 finalizeMaintainer =
     T.maintainer ~?= Just haskellMaintainer
 
-finalizeControl :: Monad m => DebT m ()
+finalizeControl :: (Monad m, Functor m) => DebT m ()
 finalizeControl =
-    do finalizeSourceName
+    do finalizeSourceName B.HaskellSource
        finalizeMaintainer
        Just src <- access T.sourcePackageName
        maint <- access T.maintainer >>= return . fromMaybe (error "No maintainer")
@@ -175,9 +183,9 @@ finalizeControl =
 -- source package name implied by the debianization.  This means
 -- either adding an entry or modifying the latest entry (if its
 -- version number is the exact one in our debianization.)
-finalizeChangelog :: Monad m => String -> DebT m ()
+finalizeChangelog :: (Monad m, Functor m) => String -> DebT m ()
 finalizeChangelog date =
-    do finalizeSourceName
+    do finalizeSourceName B.HaskellSource
        finalizeMaintainer
        ver <- debianVersion
        src <- access T.sourcePackageName
@@ -206,10 +214,10 @@ finalizeChangelog date =
 -- | Convert the extraLibs field of the cabal build info into debian
 -- binary package names and make them dependendencies of the debian
 -- devel package (if there is one.)
-addExtraLibDependencies :: (Monad m, Functor m) => DebT m ()
-addExtraLibDependencies =
+addExtraLibDependencies :: (Monad m, Functor m) => CompilerFlavor -> DebT m ()
+addExtraLibDependencies hc =
     do pkgDesc <- access T.packageDescription >>= maybe (error "addExtraLibDependencies: no PackageDescription") return
-       devName <- debianName B.Development
+       devName <- debianName B.Development hc
        libMap <- access T.extraLibMap
        binNames <- List.map (getL B.package) <$> access T.binaryPackages
        when (any (== devName) binNames) (T.depends devName %= \ deps -> deps ++ g pkgDesc libMap)
@@ -219,7 +227,7 @@ addExtraLibDependencies =
       devDep :: Map String Relations -> String -> Relations
       devDep libMap cab = maybe [[Rel (BinPkgName ("lib" ++ cab ++ "-dev")) Nothing Nothing]] id (Map.lookup cab libMap)
 
-putBuildDeps :: MonadIO m => PackageDescription -> DebT m ()
+putBuildDeps :: (MonadIO m, Functor m) => PackageDescription -> DebT m ()
 putBuildDeps pkgDesc =
     do deps <- debianBuildDeps pkgDesc
        depsIndep <- debianBuildDepsIndep pkgDesc
@@ -250,26 +258,26 @@ binaryPackageRelations b typ =
        T.replaces b %= \ rels -> [anyrel "${haskell:Replaces}"] ++ rels
        T.builtUsing b ~= []
 
-librarySpecs :: Monad m => PackageDescription -> DebT m ()
-librarySpecs pkgDesc =
-    do debName <- debianName B.Documentation
+-- | Add the library paragraphs for a particular compiler flavor.
+librarySpecs :: (Monad m, Functor m) => PackageDescription -> CompilerFlavor -> DebT m ()
+librarySpecs pkgDesc hc =
+    do debName <- debianName B.Documentation hc
        let dev = isJust (Cabal.library pkgDesc)
        doc <- get >>= return . not . getL T.noDocumentationLibrary
        prof <- get >>= return . not . getL T.noProfilingLibrary
-       cfl <- access A.compilerFlavor
        hoogle <- get >>= return . not . getL T.noHoogle
-       when dev (librarySpec Any B.Development)
-       when (dev && prof && cfl == GHC) (librarySpec Any B.Profiling)
+       when dev (librarySpec Any B.Development hc)
+       when (dev && prof && hc == GHC) (librarySpec Any B.Profiling hc)
        when (dev && doc && hoogle)
-            (do docSpecsParagraph
-                T.link +++= (debName, singleton ("/usr/share/doc" </> show (pretty debName) </> "html" </> cabal <.> "txt",
-                                                 "/usr/lib/ghc-doc/hoogle" </> List.map toLower cabal <.> "txt")))
+            (do docSpecsParagraph hc
+                T.link debName ("/usr/share/doc" </> show (pretty debName) </> "html" </> cabal <.> "txt")
+                               ("/usr/lib/ghc-doc/hoogle" </> List.map toLower cabal <.> "txt"))
     where
       PackageName cabal = pkgName (Cabal.package pkgDesc)
 
-docSpecsParagraph :: Monad m => DebT m ()
-docSpecsParagraph =
-    do b <- debianName B.Documentation
+docSpecsParagraph :: (Monad m, Functor m) => CompilerFlavor -> DebT m ()
+docSpecsParagraph hc =
+    do b <- debianName B.Documentation hc
        binaryPackageRelations b B.Development -- not sure why this isn't Documentation, but I think there's a "good" reason
        T.packageType b ~?= Just B.Documentation
        desc <- describe b
@@ -278,9 +286,9 @@ docSpecsParagraph =
        T.binarySection b ~?= Just (MainSection "doc")
        T.debianDescription b ~?= Just desc
 
-librarySpec :: Monad m => PackageArchitectures -> B.PackageType -> DebT m ()
-librarySpec arch typ =
-    do b <- debianName typ
+librarySpec :: (Monad m, Functor m) => PackageArchitectures -> B.PackageType -> CompilerFlavor -> DebT m ()
+librarySpec arch typ hc =
+    do b <- debianName typ hc
        binaryPackageRelations b B.Development
        T.packageType b ~?= Just typ
        desc <- describe b
@@ -294,17 +302,21 @@ librarySpec arch typ =
 -- files, assign them to the packages returned by the
 -- utilsPackageNames lens, and make sure those packages are in the
 -- source deb description.
-makeUtilsPackages :: forall m. (Monad m, Functor m) => PackageDescription -> DebT m ()
-makeUtilsPackages pkgDesc =
+makeUtilsPackage :: forall m. (Monad m, Functor m) => PackageDescription -> CompilerFlavor -> DebT m ()
+makeUtilsPackage pkgDesc hc =
     do -- Files the cabal package expects to be installed
        -- Files that are already assigned to any binary deb
-       installedDataMap <- Map.unionsWith Set.union
-                           <$> (sequence [(Map.map (Set.map fst) <$> access T.install),
-                                          (Map.map (Set.map fst) <$> access T.installTo),
-                                          (Map.map (Set.map fst) <$> access T.installData)]) :: DebT m (Map BinPkgName (Set FilePath))
-       installedExecMap <- Map.unionsWith Set.union
-                           <$> (sequence [(Map.map (Set.map fst) <$> access T.installCabalExec),
-                                          (Map.map (Set.map fst) <$> access T.installCabalExecTo)]) :: DebT m (Map BinPkgName (Set String))
+       installedDataMap <- Set.fold (\ x r ->
+                                         case x of
+                                           A.Install b from _ -> Map.insertWith Set.union b (singleton from) r
+                                           A.InstallTo b from _ -> Map.insertWith Set.union b (singleton from) r
+                                           A.InstallData b from _ -> Map.insertWith Set.union b (singleton from) r
+                                           _ -> r) mempty <$> access A.atomSet :: DebT m (Map BinPkgName (Set FilePath))
+       installedExecMap <- Set.fold (\ x r ->
+                                         case x of
+                                           A.InstallCabalExec b name _ -> Map.insertWith Set.union b (singleton name) r
+                                           A.InstallCabalExecTo b name _ -> Map.insertWith Set.union b (singleton name) r
+                                           _ -> r) mempty <$> access A.atomSet :: DebT m (Map BinPkgName (Set String))
 
        -- The names of cabal executables that go into eponymous debs
        insExecPkg <- access T.executable >>= return . Set.map ename . Set.fromList . elems
@@ -318,44 +330,47 @@ makeUtilsPackages pkgDesc =
        let availableData = Set.union installedData dataFilePaths
            availableExec = Set.union installedExec execFilePaths
 
-       access T.utilsPackageNames >>= \ names ->
-           when (Set.null names) (debianName B.Utilities >>= \ name -> T.utilsPackageNames ~= singleton name)
-       utilsPackages <- access T.utilsPackageNames
+       access T.utilsPackageNameBase >>= \ name ->
+           case name of
+             Nothing -> debianName B.Utilities hc >>= \ (BinPkgName name') -> T.utilsPackageNameBase ~= Just name'
+             _ -> return ()
+       b <- debianName B.Utilities hc
 
        -- Files that are installed into packages other than the utils packages
-       let installedDataOther = Set.map (\ a -> (a, a)) $ Set.unions $ Map.elems $ foldr (Map.delete) installedDataMap (Set.toList utilsPackages)
-           installedExecOther =
-               Set.union (tr "insExecPkg: " insExecPkg) $
-                                Set.unions $ Map.elems $ foldr (Map.delete) (tr "installedExec: " installedExecMap) (Set.toList utilsPackages)
+       let installedDataOther = Set.map (\ a -> (a, a)) $ Set.unions $ Map.elems $ Map.delete b installedDataMap
+           installedExecOther = Set.union insExecPkg $ Set.unions $ Map.elems $ Map.delete b installedExecMap
 
        -- Files that will be in utils packages
        let utilsData = Set.difference availableData installedDataOther
-           utilsExec = Set.difference (tr "availableExec: " availableExec) (tr "installedExecOther: " installedExecOther)
+           utilsExec = Set.difference availableExec installedExecOther
        -- Files that still need to be assigned to the utils packages
        let utilsDataMissing = Set.difference utilsData installedData
            utilsExecMissing = Set.difference utilsExec installedExec
        -- If any files belong in the utils packages, make sure they exist
-       when (not (Set.null utilsData && Set.null utilsExec))
-            (Set.mapM_ (\ p -> do -- This is really for all binary debs except the libraries - I'm not sure why
-                                  T.rulesFragments += (pack ("build" </> show (pretty p) ++ ":: build-$(HC)-stamp"))
-                                  T.binaryArchitectures p ~?= Just (if Set.null utilsExec then All else Any)
-                                  T.binarySection p ~?= Just (MainSection "misc")
-                                  binaryPackageRelations p B.Utilities) utilsPackages)
+       when (not (Set.null utilsData && Set.null utilsExec)) $ do
+         -- This is really for all binary debs except the libraries - I'm not sure why
+         T.rulesFragments += (pack ("build" </> show (pretty b) ++ ":: build-ghc-stamp"))
+         T.binaryArchitectures b ~?= Just (if Set.null utilsExec then All else Any)
+         T.binarySection b ~?= Just (MainSection "misc")
+         binaryPackageRelations b B.Utilities
        -- Add the unassigned files to the utils packages
-       Set.mapM_ (\ p -> Set.mapM_ (\ pair -> T.installData +++= (p, singleton pair)) utilsDataMissing) utilsPackages
-       Set.mapM_ (\ p -> Set.mapM_ (\ name -> T.installCabalExec +++= (p, singleton (name, "usr/bin"))) (tr "utilsExecMissing: " utilsExecMissing)) utilsPackages
+       Set.mapM_ (uncurry (T.installData b)) utilsDataMissing
+       Set.mapM_ (\ name -> T.installCabalExec b name "usr/bin") utilsExecMissing
     where
       ename i =
           case A.sourceDir i of
             (Nothing) -> A.execName i
             (Just s) ->  s </> A.execName i
 
-tr :: Show a => String -> a -> a
-tr _label x = {- trace ("(trace " ++ _label ++ show x ++ ")") -} x
-
 expandAtoms :: Monad m => DebT m ()
 expandAtoms =
-    do builddir <- get >>= return . fromEmpty (singleton "dist-ghc/build") . getL T.buildDir
+    do hcs <- access A.compilerFlavors >>= return . Set.toList
+       builddir <- access T.buildDir >>= return . fromEmpty (case hcs of
+                                                               [GHC] -> singleton "dist-ghc/build"
+#if MIN_VERSION_Cabal(1,21,0)
+                                                               [GHCJS] -> singleton "dist-ghcjs/build"
+#endif
+                                                               _ -> error $ "Unexpected compiler: " ++ show hcs)
        dDir <- access T.packageDescription >>= maybe (error "expandAtoms") (return . dataDir)
        expandApacheSites
        expandInstallCabalExecs (fromSingleton (error "no builddir") (\ xs -> error $ "multiple builddirs:" ++ show xs) builddir)
@@ -374,46 +389,76 @@ expandAtoms =
              List.mapM_ expandApacheSite (Map.toList mp)
           where
             expandApacheSite (b, (dom, log, text)) =
-                do T.link +++= (b, singleton ("/etc/apache2/sites-available/" ++ dom, "/etc/apache2/sites-enabled/" ++ dom))
-                   T.installDir +++= (b, singleton log)
-                   T.file +++= (b, singleton ("/etc/apache2/sites-available" </> dom, text))
+                do T.link b ("/etc/apache2/sites-available/" ++ dom) ("/etc/apache2/sites-enabled/" ++ dom)
+                   T.installDir b log
+                   T.file b ("/etc/apache2/sites-available" </> dom) text
 
+      -- Turn A.InstallCabalExec into A.Install
       expandInstallCabalExecs :: Monad m => FilePath -> DebT m ()
-      expandInstallCabalExecs builddir =
-          do mp <- get >>= return . getL T.installCabalExec
-             List.mapM_ (\ (b, pairs) -> Set.mapM_ (\ (name, dst) -> T.install +++= (b, singleton (builddir </> name </> name, dst))) pairs) (Map.toList mp)
+      expandInstallCabalExecs builddir = do
+        hcs <- access A.compilerFlavors >>= return . Set.toList
+        access A.atomSet >>= List.mapM_ (doAtom hcs) . Set.toList
+          where
+            doAtom [GHC] (A.InstallCabalExec b name dest) = T.install b (builddir </> name </> name) dest
+            -- A GHCJS executable is a directory with files, copy them
+            -- all into place.
+#if MIN_VERSION_Cabal(1,21,0)
+            doAtom [GHCJS] (A.InstallCabalExec b name dest) =
+                T.rulesFragments +=
+                     (Text.unlines
+                        [ pack ("binary-fixup" </> show (pretty b)) <> "::"
+                        , pack ("\t(cd " <> builddir </> name <> " && find " <> name <.> "jsexe" <> " -type f) |\\\n" <>
+                                       "\t  while read i; do install -Dp " <> builddir </> name </> "$$i debian" </> show (pretty b) </> makeRelative "/" dest </> "$$i; done") ])
+#endif
+            doAtom _ _ = return ()
 
+      -- Turn A.InstallCabalExecTo into a make rule
       expandInstallCabalExecTo :: Monad m => FilePath -> DebT m ()
-      expandInstallCabalExecTo builddir =
-          do mp <- get >>= return . getL T.installCabalExecTo
-             List.mapM_ (\ (b, pairs) -> Set.mapM_ (\ (n, d) -> T.rulesFragments += (Text.unlines
-                                                                                     [ pack ("binary-fixup" </> show (pretty b)) <> "::"
-                                                                                     , "\tinstall -Dps " <> pack (builddir </> n </> n) <> " " <> pack ("debian" </> show (pretty b) </> makeRelative "/" d) ])) pairs) (Map.toList mp)
+      expandInstallCabalExecTo builddir = do
+        hcs <- access A.compilerFlavors >>= return . Set.toList
+        access A.atomSet >>= List.mapM_ (doAtom hcs) . Set.toList
+          where
+            doAtom [GHC] (A.InstallCabalExecTo b name dest) =
+                T.rulesFragments += (Text.unlines
+                                       [ pack ("binary-fixup" </> show (pretty b)) <> "::"
+                                       , "\tinstall -Dps " <> pack (builddir </> name </> name) <> " "
+                                                           <> pack ("debian" </> show (pretty b) </> makeRelative "/" dest) ])
+            doAtom hcs (A.InstallCabalExecTo b name dest) = error $ "expandInstallCabalExecTo " ++ show hcs ++ " " ++ show (A.InstallCabalExecTo b name dest)
+            doAtom _ _ = return ()
 
+      -- Turn A.InstallData into either an Install or an InstallTo
       expandInstallData :: Monad m => FilePath -> DebT m ()
       expandInstallData dDir =
-          do mp <- get >>= return . getL T.installData
-             List.mapM_ (\ (b, pairs) -> Set.mapM_ (\ (s, d) ->
-                                                        if takeFileName s == takeFileName d
-                                                        then T.install +++= (b, singleton (s, (dDir </> makeRelative "/" (takeDirectory d))))
-                                                        else T.installTo +++= (b, singleton (s, (dDir </> makeRelative "/" d)))) pairs) (Map.toList mp)
+          access A.atomSet >>= List.mapM_ doAtom . Set.toList
+          where
+            doAtom (A.InstallData b from dest) =
+                if takeFileName from == takeFileName dest
+                then T.install b from (dDir </> makeRelative "/" (takeDirectory dest))
+                else T.installTo b from (dDir </> makeRelative "/" dest)
+            doAtom _ = return ()
 
+      -- Turn A.InstallTo into a make rule
       expandInstallTo :: Monad m => DebT m ()
       expandInstallTo =
-          do mp <- get >>= return . getL T.installTo
-             List.mapM_ (\ (p, pairs) -> Set.mapM_ (\ (s, d) -> T.rulesFragments += (Text.unlines
-                                                                                     [ pack ("binary-fixup" </> show (pretty p)) <> "::"
-                                                                                     , "\tinstall -Dp " <> pack s <> " " <> pack ("debian" </> show (pretty p) </> makeRelative "/" d) ])) pairs) (Map.toList mp)
+          access A.atomSet >>= List.mapM_ doAtom . Set.toList
+          where
+            doAtom (A.InstallTo b from dest) =
+                T.rulesFragments += (Text.unlines [ pack ("binary-fixup" </> show (pretty b)) <> "::"
+                                                  , "\tinstall -Dp " <> pack from <> " " <> pack ("debian" </> show (pretty b) </> makeRelative "/" dest) ])
+            doAtom _ = return ()
 
+      -- Turn A.File into an intermediateFile and an A.Install
       expandFile :: Monad m => DebT m ()
       expandFile =
-          do mp <- get >>= return . getL T.file
-             List.mapM_ (\ (p, pairs) -> Set.mapM_ (\ (path, s) ->
-                                                        do let (destDir', destName') = splitFileName path
-                                                               tmpDir = "debian/cabalInstall" </> show (md5 (fromString (unpack s)))
-                                                               tmpPath = tmpDir </> destName'
-                                                           T.intermediateFiles += (tmpPath, s)
-                                                           T.install +++= (p, singleton (tmpPath, destDir'))) pairs) (Map.toList mp)
+          access A.atomSet >>= List.mapM_ doAtom . Set.toList
+          where
+            doAtom (A.File b path text) =
+                do let (destDir', destName') = splitFileName path
+                       tmpDir = "debian/cabalInstall" </> show (md5 (fromString (unpack text)))
+                       tmpPath = tmpDir </> destName'
+                   T.intermediateFiles += (tmpPath, text)
+                   T.install b tmpPath destDir'
+            doAtom _ = return ()
 
       expandWebsite :: Monad m => DebT m ()
       expandWebsite =
