@@ -1,6 +1,7 @@
 -- | Things that seem like they could be clients of this library, but
 -- are instead included as part of the library.
 {-# LANGUAGE OverloadedStrings #-}
+{-# OPTIONS_GHC -fno-warn-unused-do-bind #-}
 module Debian.Debianize.Goodies
     ( tightDependencyFixup
     , doServer
@@ -24,14 +25,15 @@ import Data.List as List (map, intersperse, intercalate)
 import Data.Map as Map (insertWith)
 import Data.Maybe (fromMaybe)
 import Data.Monoid ((<>))
-import Data.Set as Set (insert, union, singleton)
+import Data.Set as Set (insert, union, singleton, toList)
 import Data.Text as Text (Text, pack, unlines, intercalate)
-import Debian.Debianize.DebianName (debianName)
+import Debian.Debianize.DebianName (debianNameBase)
 import Debian.Debianize.Monad (Atoms, DebT, execDebM)
 import Debian.Debianize.Prelude (trim, (%=), (+=), (++=), (+++=))
 import qualified Debian.Debianize.Types as T
 import qualified Debian.Debianize.Types.Atoms as T
 import qualified Debian.Debianize.Types.BinaryDebDescription as B
+import Debian.Debianize.VersionSplits (DebBase(DebBase))
 import Debian.Orphans ()
 import Debian.Pretty (Pretty(pretty))
 import Debian.Policy (apacheLogDirectory, apacheErrorLog, apacheAccessLog, databaseDirectory, serverAppLog, serverAccessLog)
@@ -173,10 +175,10 @@ watchAtom (PackageName pkgname) =
 siteAtoms :: BinPkgName -> T.Site -> Atoms -> Atoms
 siteAtoms b site =
     execDebM
-      (do T.installDir +++= (b, singleton "/etc/apache2/sites-available")
-          T.link +++= (b, singleton ("/etc/apache2/sites-available/" ++ T.domain site, "/etc/apache2/sites-enabled/" ++ T.domain site))
-          T.file +++= (b, singleton ("/etc/apache2/sites-available" </> T.domain site, apacheConfig))
-          T.installDir +++= (b, singleton (apacheLogDirectory b))
+      (do T.installDir b "/etc/apache2/sites-available"
+          T.link b ("/etc/apache2/sites-available/" ++ T.domain site) ("/etc/apache2/sites-enabled/" ++ T.domain site)
+          T.file b ("/etc/apache2/sites-available" </> T.domain site) apacheConfig
+          T.installDir b (apacheLogDirectory b)
           T.logrotateStanza +++= (b, singleton
                                    (Text.unlines $ [ pack (apacheAccessLog b) <> " {"
                                                    , "  copytruncate" -- hslogger doesn't notice when the log is rotated, maybe this will help
@@ -335,7 +337,7 @@ backupAtoms b name =
 -- FIXME - use Atoms
 execAtoms :: BinPkgName -> T.InstallFile -> Atoms -> Atoms
 execAtoms b ifile r =
-    modL T.rulesFragments (Set.insert (pack ("build" </> show (pretty b) ++ ":: build-$(HC)-stamp"))) .
+    modL T.rulesFragments (Set.insert (pack ("build" </> show (pretty b) ++ ":: build-ghc-stamp"))) .
     fileAtoms b ifile $
     r
 
@@ -348,21 +350,28 @@ fileAtoms b installFile' r =
 fileAtoms' :: BinPkgName -> Maybe FilePath -> String -> Maybe FilePath -> String -> Atoms -> Atoms
 fileAtoms' b sourceDir' execName' destDir' destName' r =
     case (sourceDir', execName' == destName') of
-      (Nothing, True) -> modL T.installCabalExec (insertWith Set.union b (singleton (execName', d))) r
-      (Just s, True) -> modL T.install (insertWith Set.union b (singleton (s </> execName', d))) r
-      (Nothing, False) -> modL T.installCabalExecTo (insertWith Set.union b (singleton (execName', (d </> destName')))) r
-      (Just s, False) -> modL T.installTo (insertWith Set.union b (singleton (s </> execName', d </> destName'))) r
+      (Nothing, True) -> execDebM (T.installCabalExec b execName' d) r
+      (Just s, True) -> execDebM (T.install b (s </> execName') d) r
+      (Nothing, False) -> execDebM (T.installCabalExecTo b execName' (d </> destName')) r
+      (Just s, False) -> execDebM (T.installTo b (s </> execName') (d </> destName')) r
     where
       d = fromMaybe "usr/bin" destDir'
 
 -- | Build a suitable value for the head of the rules file.
-makeRulesHead :: Monad m => DebT m Text
+makeRulesHead :: (Monad m, Functor m) => DebT m Text
 makeRulesHead =
-    do b <- debianName B.Cabal
-       hc <- access T.compilerFlavor
-       let ls = ["DEB_CABAL_PACKAGE = " <> pack (show (pretty (b :: BinPkgName))),
-                 "HC = " <> pack (map toLower (show hc)),
-                 ""]
+    do DebBase b <- debianNameBase
+       compilers <- access T.compilerFlavors
+       let ls = ["DEB_CABAL_PACKAGE = " <> pack b] ++
+                -- If there is a single entry in the compiler list we
+                -- can tell debian/rules what compiler to use,
+                -- otherwise it has to figure this out from the
+                -- package names in debian/control.  This is required
+                -- to build a package without a library.
+                (case toList compilers of
+                   [x] -> ["DEB_DEFAULT_COMPILER = " <> pack (map toLower (show x))]
+                   _ -> []) ++
+                [""]
        return $
           Text.unlines $
             ["#!/usr/bin/make -f", ""] ++
