@@ -2,6 +2,7 @@
 -- record, the record that holds the input data from which the
 -- debianization is to be constructed.
 {-# LANGUAGE CPP, DeriveDataTypeable #-}
+{-# OPTIONS_GHC -Wall #-}
 module Debian.Debianize.Types.Atoms
 {-    ( Atoms
     , showAtoms
@@ -16,11 +17,13 @@ module Debian.Debianize.Types.Atoms
 
 import Control.Applicative ((<$>))
 import Control.Category ((.))
+import Control.Monad.State (StateT)
 import Control.Monad.Trans (MonadIO, liftIO)
-import Data.Lens.Lazy (Lens, lens)
+import Data.Generics (Data, Typeable)
+import Data.Lens.Lazy (Lens, lens, (%=))
 import Data.Map as Map (Map)
 import Data.Monoid (Monoid(..))
-import Data.Set as Set (Set)
+import Data.Set as Set (Set, singleton, insert)
 import Data.Text (Text)
 import Debian.Changes (ChangeLog)
 import qualified Debian.Debianize.Types.SourceDebDescription as S
@@ -29,7 +32,7 @@ import Debian.Orphans ()
 import Debian.Policy (PackageArchitectures, PackagePriority, Section, SourceFormat)
 import Debian.Relation (BinPkgName, Relations, SrcPkgName)
 import Debian.Version (DebianVersion)
-import Distribution.Compiler (CompilerFlavor)
+import Distribution.Compiler (CompilerFlavor(GHC))
 import Distribution.License (License)
 import Distribution.Package (PackageName)
 import Distribution.PackageDescription as Cabal (FlagName, PackageDescription)
@@ -39,6 +42,14 @@ import System.Environment (getArgs)
 import System.FilePath ((</>))
 import Text.ParserCombinators.Parsec.Rfc2822 (NameAddr)
 
+-- This enormous record is a mistake - instead it should be an Atom
+-- type with lots of constructors, and the Atoms type is a set of
+-- these.  Then we can cruise through the atom set converting the
+-- elements into other simpler elements until they elements are all
+-- simple enough to convert directly into a debianization.  At the
+-- moment I really need this for the Install atoms, so I will try to
+-- convert just that portion of the type to this new scheme.
+
 -- | Bits and pieces of information about the mapping from cabal package
 -- names and versions to debian package names and versions.  In essence,
 -- an 'Atoms' value represents a package's debianization.  The lenses in
@@ -119,7 +130,7 @@ data Atoms
       -- ^ A Fragment of debian/rules
       , warning_ :: Set Text
       -- ^ A warning to be reported later
-      , utilsPackageNames_ :: Set BinPkgName
+      , utilsPackageNameBase_ :: Maybe String
       -- ^ Name of a package that will get left-over data files and executables.
       -- If there are more than one, each package will get those files.
       , changelog_ :: Maybe ChangeLog
@@ -157,8 +168,6 @@ data Atoms
       -- ^ Have Apache configure a site using PACKAGE, DOMAIN, LOGDIR, and APACHECONFIGFILE
       , logrotateStanza_ :: Map BinPkgName (Set Text)
       -- ^ Add a stanza of a logrotate file to the binary package
-      , link_ :: Map BinPkgName (Set (FilePath, FilePath))
-      -- ^ Create a symbolic link in the binary package
       , postInst_ :: Map BinPkgName Text
       -- ^ Script to run after install, should contain #DEBHELPER# line before exit 0
       , postRm_ :: Map BinPkgName Text
@@ -179,6 +188,9 @@ data Atoms
       -- ^ Set the Section field of the source package
       , binarySections_ :: Map BinPkgName Section
       -- ^ Set the Section field of a binary package
+#if 0
+      , link_ :: Map BinPkgName (Set (FilePath, FilePath))
+      -- ^ Create a symbolic link in the binary package
       , install_ :: Map BinPkgName (Set (FilePath, FilePath))
       -- ^ Install a build file into the binary package
       , installTo_ :: Map BinPkgName (Set (FilePath, FilePath))
@@ -193,6 +205,9 @@ data Atoms
       -- ^ Install a cabal executable into the binary package at an exact location
       , installDir_ :: Map BinPkgName (Set FilePath)
       -- ^ Create a directory in the binary package
+#else
+      , atomSet_ :: Set Atom
+#endif
       , installInit_ :: Map BinPkgName Text
       -- ^ Add an init.d file to the binary package
       , executable_ :: Map BinPkgName InstallFile
@@ -208,31 +223,50 @@ data Atoms
       -- reason to use this is because we don't yet know the name of the dev library package.
       , packageDescription_ :: Maybe PackageDescription
       -- ^ The result of reading a cabal configuration file.
-      , compilerFlavor_ :: CompilerFlavor
-      -- ^ The newest available version of GHC in the build repository.  We don't support
-      -- base repositories with no version of GHC, it has been standard in Debian and
-      -- Ubuntu for quite some time.
-      } deriving (Eq, Show)
+      , compilerFlavors_ :: Set CompilerFlavor
+      -- ^ Which compilers should we generate library packages for?
+      } deriving (Eq, Show, Data, Typeable)
+
+data Atom
+    = Link BinPkgName FilePath FilePath
+      -- ^ Create a symbolic link in the binary package
+    | Install BinPkgName FilePath FilePath
+      -- ^ Install a build file into the binary package
+    | InstallTo BinPkgName FilePath FilePath
+      -- ^ Install a build file into the binary package at an exact location
+    | InstallData BinPkgName FilePath FilePath
+      -- ^ DHInstallTo somewhere relative to DataDir (see above)
+    | File BinPkgName FilePath Text
+      -- ^ Create a file with the given text at the given path
+    | InstallCabalExec BinPkgName String FilePath
+      -- ^ Install a cabal executable into the binary package
+    | InstallCabalExecTo BinPkgName String FilePath
+      -- ^ Install a cabal executable into the binary package at an exact location
+    | InstallDir BinPkgName FilePath
+      -- ^ Create a directory in the binary package
+    deriving (Show, Eq, Ord, Data, Typeable)
+
+type Atoms' = Set Atom
 
 data EnvSet = EnvSet
     { cleanOS :: FilePath  -- ^ The output of the debootstrap command
     , dependOS :: FilePath -- ^ An environment with build dependencies installed
     , buildOS :: FilePath  -- ^ An environment where we have built a package
-    } deriving (Eq, Show)
+    } deriving (Eq, Show, Data, Typeable)
 
 -- | Look for --buildenvdir in the command line arguments to get the
 -- changeroot path, use "/" if not present.
-newAtoms :: MonadIO m => CompilerFlavor -> m Atoms
-newAtoms hc = liftIO $ do
+newAtoms :: MonadIO m => m Atoms
+newAtoms = liftIO $ do
   (roots, _, _) <- getOpt Permute [Option "buildenvdir" [] (ReqArg id "PATH")
                                           "Directory containing the build environment"] <$> getArgs
   let envset = case roots of
                  (x : _) -> EnvSet {cleanOS = x </> "clean", dependOS = x </> "depend", buildOS = x </> "build"}
                  _ -> EnvSet {cleanOS = "/", dependOS = "/", buildOS = "/"}
-  return $ makeAtoms hc envset
+  return $ makeAtoms envset
 
-makeAtoms :: CompilerFlavor -> EnvSet -> Atoms
-makeAtoms hc envset =
+makeAtoms :: EnvSet -> Atoms
+makeAtoms envset =
     Atoms
       { noDocumentationLibrary_ = False
       , noProfilingLibrary_ = False
@@ -254,7 +288,7 @@ makeAtoms hc envset =
       , rulesHead_ = Nothing
       , rulesFragments_ = mempty
       , warning_ = mempty
-      , utilsPackageNames_ = mempty
+      , utilsPackageNameBase_ = Nothing
       , changelog_ = Nothing
       , comments_ = Nothing
       , missingDependencies_ = mempty
@@ -268,7 +302,6 @@ makeAtoms hc envset =
       , licenseFile_ = mempty
       , apacheSite_ = mempty
       , logrotateStanza_ = mempty
-      , link_ = mempty
       , postInst_ = mempty
       , postRm_ = mempty
       , preInst_ = mempty
@@ -279,6 +312,8 @@ makeAtoms hc envset =
       , binaryPriorities_ = mempty
       , sourceSection_ = Nothing
       , binarySections_ = mempty
+#if 0
+      , link_ = mempty
       , install_ = mempty
       , installTo_ = mempty
       , installData_ = mempty
@@ -286,6 +321,9 @@ makeAtoms hc envset =
       , installCabalExec_ = mempty
       , installCabalExecTo_ = mempty
       , installDir_ = mempty
+#else
+      , atomSet_ = mempty
+#endif
       , installInit_ = mempty
       , executable_ = mempty
       , serverInfo_ = mempty
@@ -293,7 +331,7 @@ makeAtoms hc envset =
       , backups_ = mempty
       , extraDevDeps_ = mempty
       , packageDescription_ = Nothing
-      , compilerFlavor_ = hc
+      , compilerFlavors_ = singleton GHC
       }
 
 -- | This record supplies information about the task we want done -
@@ -317,17 +355,17 @@ data Flags = Flags
     -- the control file must match.
     , debAction_ :: DebAction
     -- ^ What to do - Usage, Debianize or Substvar
-    } deriving (Eq, Ord, Show)
+    } deriving (Eq, Ord, Show, Data, Typeable)
 
-data DebAction = Usage | Debianize | SubstVar DebType deriving (Read, Show, Eq, Ord)
+data DebAction = Usage | Debianize | SubstVar DebType deriving (Read, Show, Eq, Ord, Data, Typeable)
 
 -- | A redundant data type, too lazy to expunge.
-data DebType = Dev | Prof | Doc deriving (Eq, Ord, Read, Show)
+data DebType = Dev | Prof | Doc deriving (Eq, Ord, Read, Show, Data, Typeable)
 
 data PackageInfo = PackageInfo { cabalName :: PackageName
                                , devDeb :: Maybe (BinPkgName, DebianVersion)
                                , profDeb :: Maybe (BinPkgName, DebianVersion)
-                               , docDeb :: Maybe (BinPkgName, DebianVersion) } deriving (Eq, Ord, Show)
+                               , docDeb :: Maybe (BinPkgName, DebianVersion) } deriving (Eq, Ord, Show, Data, Typeable)
 
 -- | Information about the web site we are packaging.
 data Site
@@ -337,7 +375,7 @@ data Site
                            -- redirect requests from this domain to hostname:port
       , serverAdmin :: String   -- ^ Apache ServerAdmin parameter
       , server :: Server   -- ^ The hint to install the server job
-      } deriving (Read, Show, Eq, Ord)
+      } deriving (Read, Show, Eq, Ord, Data, Typeable)
 
 -- | Information about the server we are packaging.
 data Server
@@ -352,7 +390,7 @@ data Server
       , retry :: String         -- ^ start-stop-daemon --retry argument
       , serverFlags :: [String] -- ^ Extra flags to pass to the server via the init script
       , installFile :: InstallFile -- ^ The hint to install the server executable
-      } deriving (Read, Show, Eq, Ord)
+      } deriving (Read, Show, Eq, Ord, Data, Typeable)
 
 data InstallFile
     = InstallFile
@@ -360,7 +398,7 @@ data InstallFile
       , sourceDir :: Maybe FilePath -- ^ where to find it, default is dist/build/<execName>/
       , destDir :: Maybe FilePath -- ^ where to put it, default is usr/bin/<execName>
       , destName :: String  -- ^ name to give installed executable
-      } deriving (Read, Show, Eq, Ord)
+      } deriving (Read, Show, Eq, Ord, Data, Typeable)
 
 defaultFlags :: Flags
 defaultFlags =
@@ -476,10 +514,8 @@ missingDependencies :: Lens Atoms (Set BinPkgName)
 missingDependencies = lens missingDependencies_ (\ a b -> b {missingDependencies_ = a})
 
 -- | Override the package name used to hold left over data files and executables.
--- Usually only one package is specified, but if more then one are they will each
--- receive the same list of files.
-utilsPackageNames :: Lens Atoms (Set BinPkgName)
-utilsPackageNames = lens utilsPackageNames_ (\ a b -> b {utilsPackageNames_ = a})
+utilsPackageNameBase :: Lens Atoms (Maybe String)
+utilsPackageNameBase = lens utilsPackageNameBase_ (\ a b -> b {utilsPackageNameBase_ = a})
 
 -- | Override the debian source package name constructed from the cabal name
 sourcePackageName :: Lens Atoms (Maybe SrcPkgName)
@@ -592,6 +628,7 @@ control = lens control_ (\ a b -> b {control_ = a})
 logrotateStanza :: Lens Atoms (Map BinPkgName (Set Text))
 logrotateStanza = lens logrotateStanza_ (\ a b -> b {logrotateStanza_ = a})
 
+#if 0
 -- | Add entries to a binary deb's debian/foo.links file.
 link :: Lens Atoms (Map BinPkgName (Set (FilePath, FilePath)))
 link = lens link_ (\ a b -> b {link_ = a})
@@ -631,6 +668,29 @@ installCabalExecTo = lens installCabalExecTo_ (\ a b -> b {installCabalExecTo_ =
 -- FIXME: change signature to BinPkgName -> Lens Atoms (Set FilePath)
 installDir :: Lens Atoms (Map BinPkgName (Set FilePath))
 installDir = lens installDir_ (\ a b -> b {installDir_ = a})
+#else
+-- | Access the set of new style atoms.
+atomSet :: Lens Atoms (Set Atom)
+atomSet = lens atomSet_ (\ a b -> b {atomSet_ = a})
+
+-- We need (%=_)
+link :: Monad m => BinPkgName -> FilePath -> FilePath -> StateT Atoms m ()
+link b from dest = atomSet %= (Set.insert $ Link b from dest) >> return ()
+install :: Monad m => BinPkgName -> FilePath -> FilePath -> StateT Atoms m ()
+install b from dest = atomSet %= (Set.insert $ Install b from dest) >> return ()
+installTo :: Monad m => BinPkgName -> FilePath -> FilePath -> StateT Atoms m ()
+installTo b from dest = atomSet %= (Set.insert $ InstallTo b from dest) >> return ()
+installData :: Monad m => BinPkgName -> FilePath -> FilePath -> StateT Atoms m ()
+installData b from dest = atomSet %= (Set.insert $ InstallData b from dest) >> return ()
+file :: Monad m => BinPkgName -> FilePath -> Text -> StateT Atoms m ()
+file b dest content = atomSet %= (Set.insert $ File b dest content) >> return ()
+installCabalExec :: Monad m => BinPkgName -> String -> FilePath -> StateT Atoms m ()
+installCabalExec b name dest = atomSet %= (Set.insert $ InstallCabalExec b name dest) >> return ()
+installCabalExecTo :: Monad m => BinPkgName -> String -> FilePath -> StateT Atoms m ()
+installCabalExecTo b name dest = atomSet %= (Set.insert $ InstallCabalExecTo b name dest) >> return ()
+installDir :: Monad m => BinPkgName -> FilePath -> StateT Atoms m ()
+installDir b dir = atomSet %= (Set.insert $ InstallDir b dir) >> return ()
+#endif
 
 -- | Create an /etc/init.d file in the package
 -- FIXME: change signature to BinPkgName -> Lens Atoms Text
@@ -642,8 +702,8 @@ installInit = lens installInit_ (\ a b -> b {installInit_ = a})
 intermediateFiles :: Lens Atoms (Set (FilePath, Text))
 intermediateFiles = lens intermediateFiles_ (\ a b -> b {intermediateFiles_ = a})
 
-compilerFlavor :: Lens Atoms CompilerFlavor
-compilerFlavor = lens compilerFlavor_ (\ a b -> b {compilerFlavor_ = a})
+compilerFlavors :: Lens Atoms (Set CompilerFlavor)
+compilerFlavors = lens compilerFlavors_ (\ a b -> b {compilerFlavors_ = a})
 
 {-
 compilerFlavor :: Monad m => StateT Atoms m CompilerFlavor
